import 'dart:async';
import 'dart:math';

import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';

import 'package:twister/cubit/game_cubit.dart';
import 'package:twister/models/game/move.dart';
import 'package:twister/ui/widgets/game/move.dart';

class GameCurrentMoveWidget extends StatefulWidget {
  const GameCurrentMoveWidget({super.key});

  @override
  State<GameCurrentMoveWidget> createState() => _GameCurrentMoveWidgetState();
}

class _GameCurrentMoveWidgetState extends State<GameCurrentMoveWidget> {
  Move? shuffledMove;

  void animate() {
    final GameCubit gameCubit = BlocProvider.of<GameCubit>(context);
    if (gameCubit.state.currentGame.animationInProgress == true) {
      return;
    }

    const interval = Duration(milliseconds: 200);
    int iterationsLeft = 10;

    gameCubit.setAnimationIsRunning(true);
    shuffledMove = null;

    setState(() {});
    Timer.periodic(
      interval,
      (Timer timer) {
        // animate with random move
        if (iterationsLeft > 1) {
          shuffledMove = Move.pickRandom();
        }

        // last iteration => clear
        if (iterationsLeft == 1) {
          shuffledMove = null;
        }

        // end: pick and keep random move
        if (iterationsLeft == 0) {
          shuffledMove = null;
          timer.cancel();

          gameCubit.pickNewMove();
          gameCubit.setAnimationIsRunning(false);
        }

        setState(() {});
        iterationsLeft--;
      },
    );
  }

  @override
  Widget build(BuildContext context) {
    return BlocBuilder<GameCubit, GameState>(
      builder: (BuildContext context, GameState gameState) {
        return GestureDetector(
          child: gameState.currentGame.animationInProgress
              ? Transform.rotate(
                  angle: 2 * pi * Random().nextDouble(),
                  child: MoveWidget(move: shuffledMove ?? Move.createEmpty()),
                )
              : MoveWidget(move: gameState.currentGame.move ?? Move.createEmpty()),
          onTap: () {
            animate();
          },
        );
      },
    );
  }
}
